# -*- coding: utf-8 -*-

UNKNOWN = 0
INVALID_TAR = 1
ACCESS_DENIED = 2
NO_PACKAGE = 3
NO_LICENSE = 4
NO_CONFIG = 5
INVALID_CONFIG = 6
MISSING_TEMPLATE = 7
NAME_MISMATCH = 8
VERSION_TOO_LOW = 9
PRIVATE_BOILERPLATE_QUOTA = PRIVATE_APP_QUOTA = 10
INVALID_WEBSITE = 11
INVALID_FILENAME = 12
NO_WEBSITE = 13
MISSING_STATIC = 14



# -*- coding: utf-8 -*-
from StringIO import StringIO
from djeese import errorcodes
from djeese.commands import BaseCommand, CommandError, LOGIN_PATH
from djeese.input_helpers import ask_boolean
from djeese.printer import Printer
from djeese.utils import is_valid_file_name
from optparse import make_option
import os
import requests
import tarfile
try:
    from tarfile import bltn_open
except ImportError: # 2.5 compatiblity
    bltn_open = file 


class TarfileBackport(tarfile.TarFile):
    def add(self, name, arcname=None, recursive=True, exclude=None, filter=None):
        """Add the file `name' to the archive. `name' may be any type of file
           (directory, fifo, symbolic link, etc.). If given, `arcname'
           specifies an alternative name for the file in the archive.
           Directories are added recursively by default. This can be avoided by
           setting `recursive' to False. `exclude' is a function that should
           return True for each filename to be excluded. `filter' is a function
           that expects a TarInfo object argument and returns the changed
           TarInfo object, if it returns None the TarInfo object will be
           excluded from the archive.
        """
        self._check("aw")

        if arcname is None:
            arcname = name

        # Exclude pathnames.
        if exclude is not None:
            import warnings
            warnings.warn("use the filter argument instead",
                    DeprecationWarning, 2)
            if exclude(name):
                self._dbg(2, "tarfile: Excluded %r" % name)
                return

        # Skip if somebody tries to archive the archive...
        if self.name is not None and os.path.abspath(name) == self.name:
            self._dbg(2, "tarfile: Skipped %r" % name)
            return

        self._dbg(1, name)

        # Create a TarInfo object from the file.
        tarinfo = self.gettarinfo(name, arcname)

        if tarinfo is None:
            self._dbg(1, "tarfile: Unsupported type %r" % name)
            return

        # Change or exclude the TarInfo object.
        if filter is not None:
            tarinfo = filter(tarinfo)
            if tarinfo is None:
                self._dbg(2, "tarfile: Excluded %r" % name)
                return

        # Append the tar header and data to the archive.
        if tarinfo.isreg():
            f = bltn_open(name, "rb")
            self.addfile(tarinfo, f)
            f.close()

        elif tarinfo.isdir():
            self.addfile(tarinfo)
            if recursive:
                for f in os.listdir(name):
                    self.add(os.path.join(name, f), os.path.join(arcname, f),
                            recursive, exclude, filter)

        else:
            self.addfile(tarinfo)


class Command(BaseCommand):
    help = 'Clone the static files from an website'
    option_list = BaseCommand.option_list + (
        make_option( '--noinput', action='store_true', dest='noinput', default=False,
            help='Do not ask for input. Always assume yes.'
        ),
    )

    def handle(self, website=None, sourcedir='static', **options):
        if not options['noinput'] and ask_boolean("Are you sure? This will override all files remotely!", default=True) == 'false':
            return
        printer = Printer(int(options['verbosity']), logfile='djeese.log')
        if not website:
            raise CommandError("You must provide the name of the website from which you want to push the static files as first argument")
        if not os.path.exists(sourcedir):
            raise CommandError("Source directory %r not found" % sourcedir)
        url = self.get_absolute_url('/api/v1/io/static/push/')
        username, password = self.get_auth(options['noinput'])
        session = requests.session()
        login_url = self.get_absolute_url(LOGIN_PATH)
        response = session.post(login_url, {'username': username, 'password': password})
        if response.status_code != 204:
            printer.error("Login failed")
            self.clear_auth()
            return
        data = {'name': website}
        files = {'static': self.build_tarball(sourcedir, printer)}
        response = session.post(url, data=data, files=files)
        if response.status_code == 204:
            printer.always("Success")
        elif response.status_code == 400:
            self.handle_bad_request(response, printer)
            printer.always("Push failed: Bad request")
        elif response.status_code == 403:
            printer.error("Authentication failed")
            printer.always("Push failed")
        elif response.status_code == 502:
            printer.error("Temporarily unavailable")
            printer.always("Push failed")
        else:
            printer.error("Unexpected response: %s" % response.status_code)
            printer.log_only(response.content)
            printer.always("Push failed, check djeese.log for more details")
    
    def handle_bad_request(self, response, printer):
        code = int(response.headers.get('X-DJEESE-ERROR-CODE', 0))
        meta = response.headers.get('X-DJEESE-ERROR-META', '')
        if code == errorcodes.UNKNOWN:
            if meta:
                printer.error('Unknown error: %s' % meta)
            else:
                printer.error('Unknown error')
        elif code == errorcodes.INVALID_TAR:
            printer.error("Invalid tar file supplied")
        elif code == errorcodes.INVALID_WEBSITE:
            printer.error("Website with name %r not found" % meta)
        elif code == errorcodes.ACCESS_DENIED:
            printer.error("Access denied")
        elif code == errorcodes.ACCESS_DENIED:
            printer.error("Access denied")
        elif code == errorcodes.INVALID_FILENAME:
            printer.error("Filename %r is not allowed" % meta)
        elif code == errorcodes.MISSING_STATIC:
            printer.error("No staticfiles found, try to update the djeese client.")
        else:
            printer.error("Unexpected error code: %s (%s)" % (code, meta))
        printer.info(response.content)
    
    def build_tarball(self, sourcedir, printer):
        buffer = StringIO()
        tarball = TarfileBackport.open(fileobj=buffer, mode='w:gz')
        def static_files_filter(tarinfo):
            if not tarinfo.isfile():
                return tarinfo
            if is_valid_file_name(os.path.basename(tarinfo.name), printer):
                return tarinfo
            return None
        tarball.add(sourcedir, filter=static_files_filter)
        tarball.close()
        buffer.seek(0)
        return buffer
    


from __future__ import with_statement
from collections import defaultdict
from djeese.apps import AppConfiguration, VALID_TYPES
from djeese.commands import BaseCommand
from djeese.input_helpers import (ask, SlugValidator, ask_boolean, ask_multi, 
    URLValidator, ask_choice, PathValidator, contrib, letterfirst)
from djeese.utils import slugify, get_package_data
import os

LICENSE_FILE_CANDIDATES = [
    'license',
    'license.txt',
    'LICENSE',
    'LICENSE.txt',
    'LICENSE.TXT',
]

def guess_path(name, modules):
    for module in modules:
        path = os.path.join(module, 'templates', name)
        if os.path.exists(path):
            return path
    return None

def guess_license_path():
    for candidate in LICENSE_FILE_CANDIDATES:
        if os.path.exists(candidate):
            return candidate
    return


class Command(BaseCommand):
    help = 'Create a djeese apps.'

    def handle(self, **options):
        config = AppConfiguration(1)
        name = contrib(config, 'app', 'name', ask, "Name", letterfirst)
        packagename = contrib(config, 'app', 'packagename', ask, 'Package name on PyPI', SlugValidator(), default=slugify(name))
        check_net = ask_boolean("Should we try to get additional information from djangopackages?", default=True)
        if check_net:
            data = get_package_data(packagename)
        else:
            data = defaultdict(lambda:None)
        modules = [p for p in os.listdir('.') if os.path.exists(os.path.join(p, '__init__.py'))]
        contrib(config, 'app', 'private', ask_boolean, "Private", default=False)
        contrib(config, 'app', 'url', ask, 'URL', default=data['url'])
        contrib(config, 'app', 'author', ask, 'Author', default=data['author'], required=False)
        contrib(config, 'app', 'author-url', ask, 'Author URL (optional)', default=data['author_url'], required=False)
        contrib(config, 'app', 'installed-apps', ask_multi, "Installed apps", minitems=1)
        contrib(config, 'app', 'version', ask, 'Version', default=data['version'])
        contrib(config, 'app', 'description', ask, 'Description (short)', default=data['description'])
        contrib(config, 'app', 'license', ask, 'License', default=data['license'])
        contrib(config, 'app', 'license-path', ask, 'Path to license file', PathValidator(), default=guess_license_path())
        contrib(config, 'app', 'translation-url', ask, 'URL to the translation page, eg transifex (optional)', URLValidator(), required=False)
        contrib(config, 'app', 'settings', ask_multi, 'Settings (optional)')
        contrib(config, 'app', 'plugins', ask_multi, 'Plugin (class) names (optional)')
        contrib(config, 'app', 'apphook', ask_multi, 'Apphook (class) names (optional)')
        for setting in config['app'].getlist('settings'):
            contrib(config, setting, 'name', ask, 'Name of the setting %r (Python)' % setting)
            contrib(config, setting, 'verbose-name', ask, 'Verbose name of the setting %r' % setting)
            contrib(config, setting, 'type', ask_choice, 'Type of the setting %r' % setting, choices=VALID_TYPES)
            contrib(config, setting, 'default', ask, 'Default value for setting %r (optional)' % setting, required=False)
            contrib(config, setting, 'required', ask_boolean, 'Is setting %r required' % setting, default=True)
            if config[setting].get('default', None):
                contrib(config, setting, 'editable', ask_boolean, 'Is setting %r editable' % setting, default=True)
        if ask_boolean("Does your application expose templates?", default=True) == 'true':
            while True:
                name = ask('Template path (eg %s/plugin.html)' % config['app']['packagename'])
                path = ask('Path to the source of the template', PathValidator(), default=guess_path(name, modules))
                config['templates'][name] = path
                if ask_boolean("Are there more templates?") == 'false':
                    break
        fname = '%s.ini' % packagename
        config.write(fname)



# -*- coding: utf-8 -*-
from djeese.smartconfig import SmartConfig
import os

REQUIRED_BOILERPLATE_KEYS = ['name', 'author', 'author-url', 'url', 'version',
                             'description', 'license', 'license-path']

def validate_boilerplate(config, printer, validate_templates=True):
    valid = True
    if 'boilerplate' not in config:
        printer.error("Section 'boilerplate' not found")
        valid = False
    else:
        if not validate_boilerplate_section(config, printer):
            valid = False
    if 'templates' not in config:
        printer.error("Section 'templates' not found")
        valid = False
    elif validate_templates:
        if not validate_template_section(config, printer):
            valid = False
    if valid:
        printer.always("Configuration valid")
    else:
        printer.always("Configuration invalid")
    return valid

def validate_template_section(config, printer):
    valid = True
    templates = config['templates'].as_dict()
    for template in templates.keys():
        if not os.path.exists(os.path.join('templates', template)):
            printer.error("Template %r not found" % template)
            valid = False
    return valid

def validate_boilerplate_section(config, printer):
    valid = True
    printer.info("Required section 'boilerplate' found")
    boilerplate = config['boilerplate']
    for required in REQUIRED_BOILERPLATE_KEYS:
        if required not in boilerplate:
            printer.error("Option '%s' not found in 'boilerplate' section" % required)
            valid = False
        else:
            printer.info("Required option '%s' found in 'boilerplate' section" % required)
    return valid


class BoilerplateConfiguration(SmartConfig):
    def validate(self, validate_templates=True):
        """
        Validate this configuration
        """
        return validate_boilerplate(self, self.printer, validate_templates)



from __future__ import with_statement
from djeese.boilerplates import BoilerplateConfiguration, VALID_TYPES
from djeese.commands import BaseCommand
from djeese.commands.createapp import guess_license_path
from djeese.input_helpers import (contrib, ask, letterfirst, PathValidator, 
    ask_multi, RegexValidator, ask_choice, ask_boolean)

class Command(BaseCommand):
    help = 'Create a djeese boilerplate'
    
    def handle(self, **options):
        config = BoilerplateConfiguration(1)
        contrib(config, 'boilerplate', 'name', ask, "Name", letterfirst)
        contrib(config, 'boilerplate', 'url', ask, 'Project URL (optional)', required=False)
        contrib(config, 'boilerplate', 'author', ask, 'Author')
        contrib(config, 'boilerplate', 'author-url', ask, 'Author URL (optional)', required=False)
        contrib(config, 'boilerplate', 'version', ask, 'Version')
        contrib(config, 'boilerplate', 'description', ask, 'Description (short)')
        contrib(config, 'boilerplate', 'license', ask, 'License')
        contrib(config, 'boilerplate', 'license-path', ask, 'Path to license file', PathValidator(), default=guess_license_path())
        print "Configuring CMS templates. Only list templates that should be selectable in the CMS"
        while True:
            name = ask('Verbose template name')
            path = ask('Path to the source of the template', PathValidator('templates'))
            config['templates'][path] = name
            if ask_boolean("Are there more templates?") == 'false':
                break

        config.write('boilerplate.ini')



from djeese.input_helpers import ask, ask_password, ask_boolean
from djeese.printer import Printer
from optparse import make_option, OptionParser
import djeese
import os
import re
import sys
import urlparse

AUTH_FILE = os.path.join(os.path.expanduser('~'), '.djeese')
LOGIN_PATH = '/api/v1/login/'

class CommandError(Exception):
    """
    Exception class indicating a problem while executing a management
    command.

    If this exception is raised during the execution of a management
    command, it will be caught and turned into a nicely-printed error
    message to the appropriate output stream (i.e., stderr); as a
    result, raising this exception (with a sensible description of the
    error) is the preferred way to indicate that something has gone
    wrong in the execution of a command.

    """
    pass


class BaseCommand(object):
    option_list = (
        make_option('-v', '--verbosity', action='store', dest='verbosity', default='1',
            type='choice', choices=['0', '1', '2', '3'],
            help='Verbosity level; 0=no output, 1=minimal output, 2=extra output, 3=all output'
        ),
    )
    help = ''
    args = ''
    
    def get_absolute_url(self, path):
        host = os.environ.get('DJEESE_HOST', 'https://control.djeese.com')
        scheme, netloc, _, query, fragment = urlparse.urlsplit(host)
        return urlparse.urlunsplit((scheme, netloc, path, query, fragment))
    
    def get_auth(self, noinput=False):
        username, password = None, None
        if os.path.exists(AUTH_FILE):
            fobj = open(AUTH_FILE)
            try:
                data = fobj.read()
            finally:
                fobj.close()
            if data.count(':') == 1:
                username, password = [bit.strip() for bit in data.split(':')]
        if not (username and password):
            username = ask("Username")
            password = ask_password("Password:")
            if noinput or ask_boolean("Save login data?", default=True) == 'true':
                fobj = open(AUTH_FILE, 'w')
                try:
                    data = fobj.write(u'%s:%s' % (username, password))
                finally:
                    fobj.close()
        return username, password
    
    def clear_auth(self):
        if os.path.exists(AUTH_FILE):
            os.remove(AUTH_FILE)
    
    def usage(self, subcommand):
        """
        Return a brief description of how to use this command, by
        default from the attribute ``self.help``.

        """
        usage = '%%prog %s [options] %s' % (subcommand, self.args)
        if self.help:
            return '%s\n\n%s' % (usage, self.help)
        else:
            return usage
    
    def create_parser(self, prog_name, subcommand):
        """
        Create and return the ``OptionParser`` which will be used to
        parse the arguments to this command.

        """
        return OptionParser(prog=prog_name,
                            usage=self.usage(subcommand),
                            version=djeese.__version__,
                            option_list=self.option_list)

    def print_help(self, prog_name, subcommand):
        """
        Print the help message for this command, derived from
        ``self.usage()``.

        """
        parser = self.create_parser(prog_name, subcommand)
        parser.print_help()
    
    def run_from_argv(self, argv):
        """
        Given arguments, run this command.
        """
        parser = self.create_parser(argv[0], argv[1])
        options, args = parser.parse_args(argv[2:])
        self.printer = Printer(int(options.verbosity))
        try:
            self.handle(*args, **options.__dict__)
        except CommandError, e:
            sys.stderr.write('%s\n' % e)
            sys.exit(1)
    
    def handle(self, *args, **options):
        """
        The actual logic of the command. Subclasses must implement
        this method.

        """
        raise NotImplementedError()



from __future__ import with_statement
from BaseHTTPServer import HTTPServer
from SimpleHTTPServer import SimpleHTTPRequestHandler
from SocketServer import ForkingMixIn
from djeese.commands import BaseCommand, CommandError
from djeese.printer import Printer
from optparse import make_option
from requests.structures import CaseInsensitiveDict
import cgi
import os
import requests
import urlparse


class StaticHandler(SimpleHTTPRequestHandler):
    """
    Proxies everything to StaticServer.proxied_to except for paths starting
    with '/static/' if those files exist in StaticServer.staticfolder, in which
    case it serves the local file.
    """
    def do_GET(self):
        # check if it's a /static/ request
        _, _, path, _, query, _ = urlparse.urlparse(self.path)
        params = cgi.parse_qs(query)
        if path == '/login/':
            return self.send_message("We are sorry, but login is not supported yet")
        if path.startswith('/static/'):
            shortpath = path[8:]
            # translate the filepath
            filepath = os.path.join(self.server.staticfolder, shortpath)
            # check if it's there
            if os.path.exists(filepath):
                return self.serve_file(filepath)
        return self.proxy(path, params=params)
    
    def send_message(self, message):
        self.send_response(200)
        self.send_header("Content-type", 'text/plain')
        self.send_header('Content-Length', len(message))
        self.end_headers()
        self.wfile.write(message)
    
    def serve_file(self, filepath):
        """
        Serve the file at `filepath`. Code mostly taken from SimpleHTTPServer.
        """
        ctype = self.guess_type(filepath)
        try:
            fobj = open(filepath, 'rb')
        except IOError:
            self.send_error(404, "File not found")
            return None
        self.send_response(200)
        self.send_header("Content-type", ctype)
        fs = os.fstat(fobj.fileno())
        self.send_header("Content-Length", str(fs[6]))
        self.send_header("Last-Modified", self.date_time_string(fs.st_mtime))
        self.end_headers()
        self.copyfile(fobj, self.wfile)
    
    def do_POST(self):
        """
        Post ALWAYS gets proxied
        """
        message = "We are sorry, but POST requests are currently not allowed"
        return self.send_message(message)
        
        
        # Parse the form data posted
        form = cgi.FieldStorage(
            fp=self.rfile, 
            headers=self.headers,
            environ={'REQUEST_METHOD':'POST',
                     'CONTENT_TYPE':self.headers['Content-Type'],
                     })
        
        data = {}
        files = {}

        # Echo back information about what was posted in the form
        for field in form.keys():
            field_item = form[field]
            if field_item.filename:
                # The field contains an uploaded file
                files[field] = field_item.file
            else:
                # Regular form value
                data[field] = field_item.value

        return self.proxy('post', data, files)
    
    def proxy(self, path, method='get', params=None, data=None, files=None):
        """
        Proxy to the remote site using requests (which is awesome)
        
        This is a tad inefficient and slow, but should work for reasonably
        sized requests.
        """
        # get scheme and netlocation from remote server
        scheme, netloc, _, _, _ = urlparse.urlsplit(self.server.proxied_to)
        # extract get params
        # add the path
        url = urlparse.urlunsplit((scheme, netloc, path, '', ''))
        requestheaders = CaseInsensitiveDict(self.headers)
        requestheaders['referer'] = self.server.last
        requestheaders['host'] = netloc
        self.server.last = url
        # get the response
        method = getattr(self.server.session, method)
        extra = {}
        if method == 'get':
            extra['allow_redirects'] = True
        response = method(url, params=params, data=data, files=files, headers=requestheaders, **extra)
        content = response.raw.read()
        length = len(content)
        self.send_response(response.status_code)
        response.headers['content-length'] = str(length)
        # kill transfer-encoding, this server doesn't support it
        if 'transfer-encoding' in response.headers:
            del response.headers['transfer-encoding']
        # for some reason, sometimes the 'content-encoding' is reported as 
        # 'gzip' but isn't, so the quickfix is to kill that header.
        # TODO: Make this not that magical
        if 'content-encoding' in response.headers:
            del response.headers['content-encoding']
        # proxy the headers we got from the initial request
        for key, value in response.headers.items():
            self.send_header(key, value)
        self.end_headers()
        # send the content
        self.wfile.write(content)
        
    def log_message(self, format, *args):
        self.server.printer.info(format % args)


class StaticServer(ForkingMixIn, HTTPServer):
    def __init__(self, server_address, RequestHandlerClass, proxied_to, staticfolder, printer):
        self.proxied_to = proxied_to
        self.staticfolder = staticfolder
        self.printer = printer
        self.session = requests.session()
        self.last = proxied_to
        HTTPServer.__init__(self, server_address, RequestHandlerClass)


class Command(BaseCommand):
    help = 'Serve staticfiles from a local folder.'
    option_list = BaseCommand.option_list + (
        make_option('-p', '--port', action='store', dest='port', default='8080',
            help='The port of the server', type=int,
        ),
    )
    

    def handle(self, url=None, staticfolder='static', **options):
        if not url:
            raise CommandError("You must provide the url to your website file as first argument")
        if not os.path.exists(staticfolder):
            raise CommandError("Static folder %r not found." % staticfolder)
        if not url.startswith(('http://', 'https://')):
            url = 'http://%s' % url
        server_address = ('', int(options['port']))
        printer = Printer(int(options['verbosity']))
        print "Open http://localhost:%s in your browser" % options['port']
        print "Use ctrl+c to stop the server"
        httpd = StaticServer(server_address, StaticHandler, url, staticfolder, printer)
        try:
            httpd.serve_forever()
        except KeyboardInterrupt:
            printer.always("Server shut down")



# -*- coding: utf-8 -*-
from __future__ import with_statement
from StringIO import StringIO
from collections import defaultdict
import os
import re
import requests
import shutil
import subprocess
import tarfile
import tempfile
import unicodedata
import urlparse
import xmlrpclib
try:
    import json
except ImportError:
    import simplejson as json
try:
    from requests import async
except RuntimeError:
    class dummy_async(object):
        def get(self, url, **kwargs):
            return requests.get(url, **kwargs)
        
        def map(self, rs):
            return rs
    async = dummy_async()


PYPI_KEYS = [
    'author',
    'author_email',
    'license',
    'version',
]

DJANGOPACKAGES_KEYS = [
    'repo_description',
]

def check_urls(urls, timeout=5):
    """
    Checks a list of URLs (asynchronously if possible)
    """
    rs = [async.get(url, timeout=5) for url in urls]
    return [(response.url, 200 <= response.status_code < 300) for response in async.map(rs)]

def _check_url(url):
    """
    Checks a single URL
    """
    return requests.get(url, timeout=5).status_code == 200

def slugify(value):
    """
    Normalizes string, converts to lowercase, removes non-alpha characters,
    and converts spaces to hyphens.
    
    Slightly adapted from django.template.default_filters.slugify
    """
    value = unicode(value)
    value = unicodedata.normalize('NFKD', value).encode('ascii', 'ignore')
    value = unicode(re.sub('[^\w\s-]', '', value).strip().lower())
    return re.sub('[-\s]+', '-', value)

def _transform_pypi(data):
    """
    Transform the data from PyPI into the data we want.
    """
    return dict([(key, value) for key, value in data.items() if value != "UNKNOWN" and key in PYPI_KEYS])

def get_pypi_data(slug):
    """
    Download and process data on PyPI for an app with given slug, if available.
    If not available, returns empty dictionary.
    """
    client = xmlrpclib.ServerProxy('http://pypi.python.org/pypi')
    try:
        releases = client.package_releases(slug)
    except OSError:
        return {}
    if not releases:
        return {}
    release = releases[0]
    data = client.release_data(slug, release)
    return _transform_pypi(data)

def _find_author_url(repourl):
    """
    Try to find the Author URL if the URL is on github or bitbucket.
    """
    scheme, netloc, path, _, _ = urlparse.urlsplit(repourl)
    if netloc in ['github.com', 'bitbucket.org']:
        newpath = '/'.join(path.split('/')[:2])
        return urlparse.urlunsplit((scheme, netloc, newpath, '', ''))
    else:
        return None

def _transform_djangopackages(data):
    """
    Process djangopackages data. 
    """
    url = data.get('repo_url', None)
    data = dict([(key, value) for key, value in data.items() if key in DJANGOPACKAGES_KEYS])
    data['description'] = data.pop('repo_description', None)
    data['url'] = url
    data['author_url'] = _find_author_url(url)
    return data

def get_djangopackages_data(slug):
    """
    Download and process data from djangopackages using their API.
    """
    url = 'http://djangopackages.com/api/v1/package/%s/' % slug
    response = requests.get(url, timeout=5)
    if response.status_code != 200:
        return {'description': None}
    data = json.loads(response.content)
    return _transform_djangopackages(data)

def get_package_data(slug):
    """
    Get package data from Djangopackages and PyPI
    """
    data = defaultdict(lambda:None)
    data.update(get_pypi_data(slug))
    data.update(get_djangopackages_data(slug))
    return data

def _bundle_app(workspace, setuppy, config):
    """
    Does the actual bundling for `bundle`.
    """
    fnull = open(os.devnull, 'w')
    try:
        subprocess.check_call(['python', setuppy, 'sdist', '-d', workspace], stdout=fnull, stderr=fnull)
    finally:
        fnull.close()
    eggfile = os.path.join(workspace, os.listdir(workspace)[0])
    bundle = StringIO()
    tarball = tarfile.open(fileobj=bundle, mode='w:gz')
    # add the egg
    tarball.add(eggfile, arcname='package.tar.gz')
    # add templates
    templates = config['templates'].as_dict()
    for arcname, fpath in templates.items():
        full_arcname = 'templates/%s' % arcname
        # backwards compatibility, check for URL:
        if fpath.startswith(('http://', 'https://')):
            response = requests.get(fpath)
            tarball.addfile(tarfile.TarInfo(full_arcname), response)
        else:
            tarball.add(fpath, arcname=full_arcname)
    # add license
    # backwards compatibility, check for old license-text option:
    if 'license-path' in config['app']:
        tarball.add(config['app']['license-path'], arcname='meta/LICENSE.txt')
    else:
        response = requests.get(config['app']['license-text'])
        tarball.addfile(tarfile.TarInfo('meta/LICENSE.txt'), response)
    # add the config
    configpath = os.path.join(workspace, 'config')
    with open(configpath, 'w') as fobj:
        config.write_file(fobj)
    tarball.add(configpath, 'meta/config.cfg')
    # close, seek, return
    tarball.close()
    bundle.seek(0)
    return bundle

def filter_static_files(tarinfo):
    if not tarinfo.isfile():
        return tarinfo
    basename = os.path.basename(tarinfo.name)
    if is_valid_file_name(basename):
        return tarinfo
    else:
        return None

def filter_template_files(tarinfo):
    if not tarinfo.isfile():
        return tarinfo
    basename = os.path.basename(tarinfo.name)
    ext = os.path.splitext(basename)[1]
    if ext == '.html':
        return tarinfo
    else:
        return None

def bundle_app(setuppy, config):
    """
    Bundles a setup.py and all other files required (templates/license) into
    a StringIO bundle (gzipped)
    """
    distdir = tempfile.mkdtemp(prefix='djeese')
    try:
        return _bundle_app(distdir, setuppy, config)
    finally:
        shutil.rmtree(distdir)
        
def _bundle_boilerplate(workspace, config):
    bundle = StringIO()
    tarball = tarfile.open(fileobj=bundle, mode='w:gz')
    # add templates
    tarball.add('templates', filter=filter_template_files)
    # add staticfiles
    tarball.add('static', filter=filter_static_files)
    # add license
    tarball.add(config['boilerplate']['license-path'], arcname='meta/LICENSE.txt')
    # add the config
    configpath = os.path.join(workspace, 'config')
    with open(configpath, 'w') as fobj:
        config.write_file(fobj)
    tarball.add(configpath, 'meta/config.cfg')
    # close, seek, return
    tarball.close()
    bundle.seek(0)
    return bundle

def bundle_boilerplate(config):
    workspace = tempfile.mkdtemp(prefix='djeese')
    try:
        return _bundle_boilerplate(workspace, config)
    finally:
        shutil.rmtree(workspace)


FILENAME_BASIC_RE = re.compile(r'^[a-zA-Z_]+[a-zA-Z0-9._-]*\.[a-zA-Z]{2,4}$')
ALLOWED_EXTENSIONS = [
    '.js',
    '.css',
    '.png',
    '.jpg',
    '.jpeg',
    '.gif',
    '.htc',
    '.scss',
    '.sass',
    '.rb',
    '.less',
    '.ico',
]

def is_valid_file_name(name, printer=None):
    always_print = printer.always if printer else lambda x: None
    if not FILENAME_BASIC_RE.match(name):
        always_print("File name %r is not a valid file name, ignoring..." % name)
        return False
    ext = os.path.splitext(name)[-1]
    if ext not in ALLOWED_EXTENSIONS:
        always_print("File extension %r is not allowed, ignoring" % ext)
        return False
    return True



# -*- coding: utf-8 -*-
from __future__ import with_statement
from djeese import errorcodes
from djeese.boilerplates import BoilerplateConfiguration
from djeese.commands import BaseCommand, CommandError, LOGIN_PATH
from djeese.printer import Printer
from djeese.utils import bundle_boilerplate
from optparse import make_option
import os
import requests


UPLOAD_PATH = '/api/v1/boilerplates/upload-bundle/'


class Command(BaseCommand):
    help = 'Upload a boilerplate.'
    option_list = BaseCommand.option_list + (
        make_option( '--noinput', action='store_true', dest='noinput', default=False,
            help='Do not ask for input. Always assume yes.'
        ),
    )

    def handle(self, boilerplatefile=None, **options):
        if not boilerplatefile:
            raise CommandError("You must provide the path to your boilerplatefile file as first argument")
        if not os.path.exists(boilerplatefile):
            raise CommandError("Could not find boilerplatefile at %r" % boilerplatefile)
        username, password = self.get_auth(options['noinput'])
        self.run(boilerplatefile, username, password, **options)

    def run(self, boilerplatefile, username, password, **options):
        printer = Printer(int(options['verbosity']), logfile='djeese.log')
        config = BoilerplateConfiguration(printer=printer)
        config.read(boilerplatefile)
        bundle = bundle_boilerplate(config) 
        boilerplatename = config['boilerplate']['name']
        response = self.upload(boilerplatename, bundle, username, password)
        if response.status_code == 201:
            printer.always("Upload successful (created)")
        elif response.status_code == 204:
            printer.always("Upload successful (updated)")
        elif response.status_code == 400:
            self.handle_bad_request(response, printer)
            printer.always("Upload failed")
        elif response.status_code == 403:
            printer.error("Authentication failed")
            printer.always("Upload failed")
            self.clear_auth()
        elif response.status_code == 502:
            printer.error("Temporarily unavailable")
            printer.always("Upload failed")
        else:
            printer.error("Unexpected response: %s" % response.status_code)
            printer.log_only(response.content)
            printer.always("Upload failed, check djeese.log for more details")
    
    def handle_bad_request(self, response, printer):
        code = int(response.headers.get('X-DJEESE-ERROR-CODE', 0))
        meta = response.headers.get('X-DJEESE-ERROR-META', '')
        if code == errorcodes.UNKNOWN:
            if meta:
                printer.error('Unknown error: %s' % meta)
            else:
                printer.error('Unknown error')
        elif code == errorcodes.INVALID_TAR:
            printer.error("Invalid tar file supplied")
        elif code == errorcodes.ACCESS_DENIED:
            printer.error("Access denied")
        elif code == errorcodes.NO_PACKAGE:
            printer.error("Package file missing from upload")
        elif code == errorcodes.NO_LICENSE:
            printer.error("License file missing from upload")
        elif code == errorcodes.NO_CONFIG:
            printer.error("Configuration missing from upload")
        elif code == errorcodes.INVALID_CONFIG:
            printer.error("Invalid configuration")
        elif code == errorcodes.MISSING_TEMPLATE:
            printer.error("Missing template %r" % meta)
        elif code == errorcodes.MISSING_STATIC:
            printer.error("Missing static %r" % meta)
        elif code == errorcodes.NAME_MISMATCH:
            printer.error("Supplied name does not match name in configuration")
        elif code == errorcodes.VERSION_TOO_LOW:
            server, supplied = meta.split(',')
            printer.error("Supplied version (%s) is not newer than version on server (%s)" % (supplied, server))
        elif code == errorcodes.PRIVATE_APP_QUOTA:
            printer.error("Cannot add new private boilerplate, please upgrade your plan")
        else:
            printer.error("Unexpected error code: %s (%s)" % (code, meta))
        printer.info(response.content)

    def upload(self, boilerplatename, bundle, username, password):
        files = {
            'bundle': bundle,
        }
        data = {
            'boilerplate': boilerplatename,
        }
        session = requests.session()
        login_url = self.get_absolute_url(LOGIN_PATH)
        response = session.post(login_url, {'username': username, 'password': password})
        if response.status_code != 204:
            return response
        target_url = self.get_absolute_url(UPLOAD_PATH)
        response = session.post(target_url, data=data, files=files)
        return response



from __future__ import with_statement
from djeese.boilerplates import BoilerplateConfiguration
from djeese.commands import BaseCommand, CommandError
import os
import sys

class Command(BaseCommand):
    help = 'Validate a djeese boilerplate.'

    def handle(self, boilerplate=None, **options):
        if not boilerplate:
            raise CommandError("You must provide the path to your boilerplate file as first argument")
        if not os.path.exists(boilerplate):
            raise CommandError("Boilerplate file %r not found." % appfile)
        boilerplatecfg = BoilerplateConfiguration(int(options['verbosity']))
        boilerplatecfg.read(boilerplate)
        if not boilerplatecfg.validate():
            sys.exit(1)



# -*- coding: utf-8 -*-
from __future__ import with_statement
import os

LEVELS = {
    4: 'LOG',
    3: 'INFO', 
    2: 'WARNING', 
    1: 'ERROR',
    -1: 'CRITICAL',
}

class Printer(object):
    """
    Helper object to print certain types of information only if the verbosity
    level given is high enough.
    """
    def __init__(self, verbosity, logfile=None):
        self.verbosity = verbosity
        self.logfile = open(logfile, 'w') if logfile else open(os.devnull, 'w')
    
    def _print(self, level, message):
        self.logfile.write('[%s]%s\n' % (LEVELS[level], message))
        if self.verbosity >= level:
            print message
    
    def info(self, message):
        """
        Print if verbosity is 3 or above
        """
        self._print(3, message)
    
    def warning(self, message):
        """
        Print if verbosity is 2 or above
        """
        self._print(2, message)
    
    def error(self, message):
        """
        Print if verbosity is 1 or above
        """
        self._print(1, message)
        
    def always(self, message):
        self._print(-1, message)
    
    def log_only(self, message):
        self._print(4, message)
    


"""
Borrowed mostly from epio-client
"""

import optparse
import os
import sys

__version__ = '0.5.1'


def get_commands():
    """
    Returns a list of all the command names that are available.

    Returns an empty list if no commands are defined.
    """
    command_dir = os.path.join(os.path.dirname(__file__), 'commands')
    try:
        return [f[:-3] for f in os.listdir(command_dir)
                if not f.startswith('_') and f.endswith('.py')]
    except OSError:
        return []


def load_command_class(name):
    full_name = 'djeese.commands.%s' % name
    __import__(full_name)
    return sys.modules[full_name].Command()


class LaxOptionParser(optparse.OptionParser):
    """
    An option parser that doesn't raise any errors on unknown options.

    This is needed because the --settings and --pythonpath options affect
    the commands (and thus the options) that are available to the user.
    """
    def error(self, msg):
        pass

    def print_help(self):
        """Output nothing.

        The lax options are included in the normal option parser, so under
        normal usage, we don't need to print the lax options.
        """
        pass

    def print_lax_help(self):
        """Output the basic options available to every command.

        This just redirects to the default print_help() behaviour.
        """
        optparse.OptionParser.print_help(self)

    def _process_args(self, largs, rargs, values):
        """
        Overrides OptionParser._process_args to exclusively handle default
        options and ignore args and other options.

        This overrides the behavior of the super class, which stop parsing
        at the first unrecognized option.
        """
        while rargs:
            arg = rargs[0]
            try:
                if arg[0:2] == "--" and len(arg) > 2:
                    # process a single long option (possibly with value(s))
                    # the superclass code pops the arg off rargs
                    self._process_long_opt(rargs, values)
                elif arg[:1] == "-" and len(arg) > 1:
                    # process a cluster of short options (possibly with
                    # value(s) for the last one only)
                    # the superclass code pops the arg off rargs
                    self._process_short_opts(rargs, values)
                else:
                    # it's either a non-default option or an arg
                    # either way, add it to the args list so we can keep
                    # dealing with options
                    del rargs[0]
                    raise Exception
            except:
                largs.append(arg)


class CommandLineClient(object):
    """
    Encapsulates all the option parser and commands for a command line client.
    """
    def __init__(self, argv=None):
        self.argv = argv or sys.argv[:]
        self.prog_name = os.path.basename(self.argv[0])
    
    def main_help_text(self):
        """
        Returns the script's main help text, as a string.
        """
        usage = [
            '',
            "Type '%s help <subcommand>' for help on a specific subcommand."
                % self.prog_name,
            ''
        ]
        usage.append('Available subcommands:')
        commands = get_commands()
        commands.sort()
        for cmd in commands:
            usage.append('  %s' % cmd)
        return '\n'.join(usage)
   
    def fetch_command(self, subcommand):
        """
        Tries to fetch the given subcommand, printing a message with the
        appropriate command called from the command line if it can't be found.
        """
        try:
            klass = load_command_class(subcommand)
        except (KeyError, ImportError):
            sys.stderr.write("Unknown command: %r\nType '%s help' for usage.\n" % \
                (subcommand, self.prog_name))
            sys.exit(1)
        return klass

    def execute(self):
        """
        Given the command-line arguments, this figures out which subcommand is
        being run, creates a parser appropriate to that command, and runs it.
        """
        # Late import so setup.py works
        from djeese.commands import BaseCommand
        parser = LaxOptionParser(usage="%prog subcommand [options] [args]",
                                 version=__version__,
                                 option_list=BaseCommand.option_list)
        try:
            options, args = parser.parse_args(self.argv)
        except:
            pass  # Ignore any option errors at this point.

        try:
            subcommand = self.argv[1]
        except IndexError:
            subcommand = 'help'  # Display help if no arguments were given.

        if subcommand == 'help':
            if len(args) > 2:
                self.fetch_command(args[2]).print_help(self.prog_name, args[2])
            else:
                parser.print_lax_help()
                sys.stderr.write(self.main_help_text() + '\n')
                sys.exit(1)
        elif self.argv[1:] == ['--version']:
            # LaxOptionParser already takes care of printing the version.
            pass
        elif self.argv[1:] == ['--help']:
            parser.print_lax_help()
            sys.stderr.write(self.main_help_text() + '\n')
        else:
            self.fetch_command(subcommand).run_from_argv(self.argv)
    

def main():
    CommandLineClient(sys.argv).execute()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from __future__ import with_statement
from djeese import errorcodes
from djeese.commands import BaseCommand, CommandError, LOGIN_PATH
from djeese.input_helpers import ask_boolean
from djeese.printer import Printer
from optparse import make_option
import requests
import tarfile
import traceback

class Command(BaseCommand):
    help = 'Clone the static files from an website'
    option_list = BaseCommand.option_list + (
        make_option( '--noinput', action='store_true', dest='noinput', default=False,
            help='Do not ask for input. Always assume yes.'
        ),
    )

    def handle(self, website=None, outputdir='static', **options):
        if not options['noinput'] and ask_boolean("Are you sure? This will override all files in %s!" % outputdir, default=True) == 'false':
            return
        printer = Printer(int(options['verbosity']), logfile='djeese.log')
        if not website:
            raise CommandError("You must provide the name of the website from which you want to clone the static files as first argument")
        url = self.get_absolute_url('/api/v1/io/static/clone/')
        username, password = self.get_auth(options['noinput']) 
        session = requests.session()
        login_url = self.get_absolute_url(LOGIN_PATH)
        response = session.post(login_url, {'username': username, 'password': password})
        if response.status_code != 204:
            printer.error("Login failed")
            self.clear_auth()
            return
        data = {'name': website}
        response = session.get(url, params=data)
        if response.status_code == 200:
            self.finish_clone(response, outputdir, printer)
        elif response.status_code == 400:
            self.handle_bad_request(response, printer)
            printer.always("Clone failed: Bad request")
        elif response.status_code == 403:
            printer.error("Authentication failed")
            printer.always("Clone failed")
        elif response.status_code == 502:
            printer.error("Temporarily unavailable")
            printer.always("Clone failed")
        else:
            printer.error("Unexpected response: %s" % response.status_code)
            printer.log_only(response.content)
            printer.always("Clone failed, check djeese.log for more details")
            
    
    def handle_bad_request(self, response, printer):
        code = int(response.headers.get('X-DJEESE-ERROR-CODE', 0))
        meta = response.headers.get('X-DJEESE-ERROR-META', '')
        if code == errorcodes.INVALID_WEBSITE:
            printer.error("Website with name %r not found" % meta)
        elif code == errorcodes.NO_WEBSITE:
            printer.error("You must supply a website name")
        else:
            printer.error("Unexpected error code: %s (%s)" % (code, meta))
        printer.log_only(response.content)
    
    def finish_clone(self, response, outputdir, printer):
        try:
            tarball = tarfile.open(mode='r|gz', fileobj=response.raw)
        except:
            printer.error("Response not a valid tar file.")
            printer.always("Clone failed")
            traceback.print_exc(printer.logfile)
            return
        tarball.extractall(outputdir)
        printer.info("Clone successful")



# -*- coding: utf-8 -*-
from __future__ import with_statement
from djeese.smartconfig import SmartConfig
from djeese.utils import check_urls
import os


VALID_TYPES = ['string', 'stringtuplelist', 'stringlist', 'boolean', 'integer',
               'staticfile', 'choice']
REQUIRED_APP_KEYS = ['name', 'packagename', 'installed-apps', 'description',
                     'license', 'url', 'version', 'private']
DEPRECATION_APP_KEYS = {
    'license-text': 'license-path',
}
EXTRA_APP_KEYS = ['settings', 'author', 'author-url', 'translation-url',
                  'plugins', 'apphooks']
REQUIRED_SETTINGS_KEYS = ['name', 'verbose-name', 'type']
EXTRA_SETTINGS_KEYS = ['default', 'required', 'editable']

def validate_app(config, validate_templates, printer):
    """
    Validate an app (AppConfiguration instance) and print errors using the
    printer (djeese.utils.Printer instance).
    """
    valid = True
    if 'app' not in config:
        printer.error("Section 'app' not found")
        valid = False
    else:
        if not validate_app_section(config, printer):
            valid = False
    if 'templates' in config and validate_templates:
        if not validate_templates_section(config, printer):
            valid = False
    settings = config['app'].getlist('settings')
    for setting in settings:
        if not validate_settings_section(config, printer, setting):
            valid = False
    if valid:
        printer.always("Configuration valid")
    else:
        printer.always("Configuration invalid")
    return valid

def validate_app_section(config, printer):
    """
    Validate the [app] section of an app configuration
    """
    valid = True
    printer.info("Required section 'app' found")
    app = config['app']
    for required in REQUIRED_APP_KEYS:
        if required not in app:
            printer.error("Option '%s' not found in 'app' section" % required)
            valid = False
        else:
            printer.info("Required option '%s' found in 'app' section" % required)
    for old, new in DEPRECATION_APP_KEYS.items():
        if old not in app and new not in app:
            printer.error("Option '%s' not found in 'app' section" % new)
            valid = False
        elif old in app:
            printer.warning("Option '%s' in 'app' section is deprecated, use '%s' instead" % (old, new)) 
    return valid

def validate_templates_section(config, printer):
    """
    Validate the [templates] section of an app configuration
    """
    valid = True
    templates = config['templates'].as_dict()
    reverse_templates = dict([(v,k) for k,v in templates.items()])
    urls = []
    paths = []
    for path in templates.values():
        if path.startswith(('http://', 'https://')):
            printer.warning("Template loading from URLs is deprecated.")
            urls.append(path)
        else:
            paths.append(path)
    if urls:
        responses = check_urls(urls)
        for url, success in responses:
            name = reverse_templates[url]
            if not success:
                printer.error("Could not load template %r from %r" % (name, url))
                valid = False
            else:
                printer.info("Successfully loaded %r from %r" % (name, url))
    for path in paths:
        if not os.path.exists(path):
            printer.warning("Template '%s' not found" % path)
            valid = False
    return valid

def validate_settings_section(config, printer, setting):
    """
    Validate the settings section of an app configuration for a setting
    """
    valid = True
    if setting not in config:
        printer.error("Could not find settings section %r" % setting)
        return False
    setting_config = config[setting]
    for required in ['verbose-name', 'name', 'type']:
        if required not in setting_config:
            printer.error("Could not find required option %r in settings section %r" % (required, setting))
            valid = False
    if 'type' in setting_config:
        typevalue = setting_config['type']
        if typevalue not in VALID_TYPES:
            valid_types = ', '.join(VALID_TYPES)
            printer.error("Setting %r type %r is not valid. Valid choices: %s" % (setting, typevalue, valid_types))
    printer.info("Settings section valid")
    return valid


class AppConfiguration(SmartConfig):
    def validate(self, validate_templates=True):
        """
        Validate this configuration
        """
        return validate_app(self, validate_templates, self.printer)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup
from djeese import __version__


INSTALL_REQUIRES = [
    'requests>=0.10',
]
try:
    import json
except ImportError:
    INSTALL_REQUIRES.append('simplejson')

CLASSIFIERS = [
    'Development Status :: 5 - Production/Stable',
    'Environment :: Console',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Topic :: Software Development',
]

setup(
    name='djeese',
    version=__version__,
    description='The command-line client for the djee.se service',
    author='The djee.se Team',
    author_email='say@djee.se',
    url='http://djeese.com/',
    packages=['djeese', 'djeese.commands'],
    license='LICENSE.txt',
    platforms=['OS Independent'],
    install_requires=INSTALL_REQUIRES,
    entry_points="""
    [console_scripts]
    djeese = djeese:main
    """,
)



# -*- coding: utf-8 -*-
from __future__ import with_statement
import getpass
import os
import re
import requests

def contrib(config, section, key, method, *args, **kwargs):
    """
    Wrapper to ask the user for input using `method` (passing `args` and
    `kwargs` to that method) and if a value is returned, set it in the `config`
    under `section` and `key`.
    """
    value = method(*args, **kwargs)
    if value:
        config[section][key] = value
    return value

class BaseValidator(object):
    """
    Base validator object.
    
    self.validate(value) calls self.check(value) and if not valid, prints
    self.message and return False. If valid, returns True
    """
    def validate(self, value):
        if not self.check(value):
            print self.message % value
            return False
        return True
    
    def check(self, value):
        return True
    
    
class PathValidator(BaseValidator):
    message = "File not found: %r"
    
    def __init__(self, base='.'):
        self.base = base
    
    def check(self, value):
        return os.path.exists(os.path.join(self.base, value))


class URLValidator(BaseValidator):
    """
    Checks that a given URL is valid and can be accessed
    """
    message = "Could not open %r."
    
    def check(self, value):
        return requests.get(value, timeout=5).status_code == 200


class RegexValidator(BaseValidator):
    """
    Checks that a given value matches a regular expression
    """
    def __init__(self, regex, message, *flags):
        self.regex = re.compile(regex, *flags)
        self.message = message
    
    def check(self, value):
        return self.regex.match(value)


class SlugValidator(RegexValidator):
    """
    Checks that a given value is a slug (alphanumeric + underscores + dashes)
    """
    def __init__(self):
        super(SlugValidator, self).__init__(r'^[-\w]+$', "May only contain alphanumeric characters and dashes")


def ask(title, *validators, **kwargs):
    """
    Ask for a (single) value, validate using validators.
    
    If default is given, it will be returned if no input is given.
    
    If required is True, None is returned if no input is given.
    """
    default = kwargs.get('default', None)
    required = kwargs.get('required', True)
    if default:
        message = '%s [%s]: ' % (title, default)
    else:
        message = '%s: ' % title
    value = raw_input(message)
    if not value and default:
        return default
    if not value and not required:
        return None
    for validator in validators:
        if not validator.validate(value):
            return ask(title, *validators, **kwargs)
    if not value and required:
        return ask(title, *validators, **kwargs)
    return value

def ask_boolean(title, default=None):
    """
    Ask for a (single) boolean value.
    """
    if default is None:
        message = '%s [y/n]: ' % title
    elif default:
        message = '%s [Y/n]: ' % title
    else:
        message = '%s [y/N]: ' % title
    value = raw_input(message)
    if not value and default is not None:
        return 'true' if default else 'false'
    elif value in ['y', 'Y']:
        return 'true'
    elif value in ['n', 'N']:
        return 'false'
    print "Please enter either 'n' or 'y'"
    ask_boolean(title, default)

def ask_multi(title, minitems=0):
    """
    Ask for (at least) minitems values.
    """
    values = []
    while True:
        value = ask('%s (leave empty for next option)' % title, required=len(values) < minitems)
        if value:
            values.append(value)
        else:
            break
    # convert to string:
    if values:
        return '\n    %s' % '\n    '.join(values)
    else:
        return None

def ask_choice(title, choices):
    """
    Make the user choose a single value from choices
    """
    choices_dict = dict([(str(index), choice) for index, choice in enumerate(choices)])
    print title
    for index, choice in enumerate(choices):
        print '(%s) %s' % (index, choice)
    value = ask("Please choose [0-%s]" % len(choices))
    while value not in choices_dict:
        print "Invalid choices"
        value = ask("Please choose [0-%s]" % len(choices))
    return choices_dict[value]

def ask_password(title):
    password = getpass.getpass(title)
    while not password:
        password = ask_password(title)
    return password

letterfirst = RegexValidator(r'^[a-zA-Z]', "Must start with a letter")



# -*- coding: utf-8 -*-
from ConfigParser import SafeConfigParser
from djeese.printer import Printer
from StringIO import StringIO


class Section(object):
    """
    Wrapper around ConfigParser to make a nicer interface
    """
    def __init__(self, parser, section):
        self.parser = parser
        self.section = section
    
    def items(self):
        """
        Return the items in this section as list of tuples
        """
        return self.parser.items(self.section)
        
    def as_dict(self):
        """
        Return the items in this section as dictionary
        """
        return dict(self.items())
    
    def get(self, item, default=None):
        """
        Get the item in this section
        """
        if item in self:
            return self[item]
        return default
        
    def getlist(self, item, default=None):
        """
        Get the item in this section as list
        """
        if item in self:
            return [line.strip() for line in self[item].splitlines() if line.strip()]
        if default:
            return default
        return []
    
    def getint(self, item, default=0):
        """
        Get the item in this section as integer
        """
        if item in self:
            return self.parser.getint(self.section, item)
        return default
    
    def getfloat(self, item, default=0.0):
        """
        Get the item in this section as float
        """
        if item in self:
            return self.parser.getfloat(self.section, item)
        return default
    
    def getboolean(self, item, default=False):
        """
        Get the item in this section as boolean
        """
        if item in self:
            return self.parser.getboolean(self.section, item)
        return default
    
    def __contains__(self, item):
        """
        Check if this section contains this item
        """
        return self.parser.has_option(self.section, item)
        
    def __getitem__(self, item):
        """
        Same as self.get, but without the ability to specify a default
        """
        return self.parser.get(self.section, item)
    
    def __setitem__(self, item, value):
        """
        Set an item in this section
        """
        self.parser.set(self.section, item, value)
        
    def __delitem__(self, item):
        """
        Remove an item from this section
        """
        self.parser.remove_option(self.section, item)

class SmartConfig(object):
    """
    Wrapper around SafeConfigParser to provide a nicer API
    """
    def __init__(self, verbosity=1, printer=None):
        self.parser = SafeConfigParser()
        self.printer = printer or Printer(verbosity)
        
    def __getitem__(self, item):
        """
        Get a section (Section instance) or create it if it doesn't exist.
        """
        if item not in self:
            self.parser.add_section(item)
        return Section(self.parser, item)
    
    def __contains__(self, item):
        """
        Check if this config has a section
        """
        return self.parser.has_section(item)
    
    def read_string(self, data):
        """
        Read the configuration from a string
        """
        sio = StringIO(data)
        sio.seek(0)
        self.parser.readfp(sio)
        
    def readfp(self, fp):
        self.parser.readfp(fp)
    
    def read(self, filepath):
        """
        Read the configuration from a filepath
        """
        self.parser.read(filepath)
    
    def write(self, filepath):
        """
        Write the configuration to a filepath
        """
        if not self.validate():
            return False
        with open(filepath, 'w') as fobj:
            self.parser.write(fobj)
        return True
    
    def write_file(self, fobj):
        if not self.validate():
            return False
        self.parser.write(fobj)
        return True



from __future__ import with_statement
from djeese.apps import AppConfiguration
from djeese.commands import BaseCommand, CommandError
import os
import sys

class Command(BaseCommand):
    help = 'Validate a djeese apps.'

    def handle(self, appfile=None, **options):
        if not appfile:
            raise CommandError("You must provide the path to your app file as first argument")
        if not os.path.exists(appfile):
            raise CommandError("App file %r not found." % appfile)
        appconfig = AppConfiguration(int(options['verbosity']))
        appconfig.read(appfile)
        if not appconfig.validate():
            sys.exit(1)
